#!/usr/bin/perl -w
use strict;

use Getopt::Long;

my ($input,$output,$data_path);

GetOptions('i=s'              => \$input,
           'o=s'              => \$output,
           'd=s'              => \$data_path,
           );

unless ($input && $output &&  $data_path){
	print <<EOF;
	Usage :	 perl aloft_predict.pl -i input -o output -d data_directory
			
			-i : input data, should be *.lof from ALoFT_annotate
			-o : output file
			-d : specify directory to read data
EOF
	exit;
}

die "Please download data files...\n" unless -e "$data_path/gene_feature.txt";

# data transformation
my $tmp_out = join("", (split("/",$input))[-1],".annotated");
open(OUT,">$tmp_out")||die;

my (@tmp,$gene,$tr,$score,@tmp_2,@tmp_3,$elm,$s1,$s2,$s3,$s4,$gene_feature,$rna_count);
my %tr_feature;

open(A,"$data_path/gene_feature.txt")||die;
while(<A>){
	if (!/^#/){
		chomp $_;
		@tmp = split /\t/,$_;
		$gene = $tmp[0];
		$tr = $tmp[2];
		
		# Get Gerp score for NSYn SNPs. 
		@tmp_2 = split /,/,$tmp[9];
		$score = 0; 
		foreach $elm(@tmp_2){
			$score += (split /:/,$elm)[2];
		}	
		
		#average Nsyn Gerp score
		if (scalar @tmp_2 >0){
			$s1 = $score/scalar @tmp_2;
		}else{
			$s1 = 'NA';
		}
		
		#percent of Nsyn in Gerp element
		if ($tmp[10] =~ /^$/){
			$s2 ='NA';
		}else{
			@tmp_3 = split /\)/,$tmp[10];
			if (scalar @tmp_3 >0){
				$s2 = (scalar @tmp_3 -1)/ scalar @tmp_2;
			}else{
				$s2 =0;
			}
		}
	
		#average Syn Gerp score
		@tmp_2 = split /,/,$tmp[11];
		$score = 0;
		foreach $elm(@tmp_2){
			$score += (split /:/,$elm)[2];
		}	
	
		if (scalar @tmp_2 >0){
			$s3 = $score/scalar @tmp_2;
		}else{
			$s3='NA';
		}
		
		#percent of Syn in Gerp element
		if ($tmp[12] =~ /^$/){
			$s4 ='NA';
		}else{
			@tmp_3 = split /\)/,$tmp[12];
			if (scalar @tmp_3 >0){
				$s4 = (scalar @tmp_3 -1 )/scalar @tmp_2;
			}else{
				$s4 = 0;
			}
		}
	
 		#unique microRNA.
		if ($tmp[14] eq "."){
			$rna_count = 'NA';
		} else{
			$rna_count = (split /:/,$tmp[14])[0];
		}
		
		$gene_feature = join("\t",@tmp[7..8],$s1,$s2,$s3,$s4,$rna_count,$tmp[15]/$tmp[3],$tmp[16]/$tmp[3],$tmp[17]/$tmp[3],$tmp[18]/$tmp[3],$tmp[19]*0.01);

		$tr_feature{$tr} = $gene_feature;
	}
}
close A;

my %net_feature;
open(B,"$data_path/network_feature.txt")||die;
while(<B>){
	chomp $_;
	@tmp = split /\t/,$_;
	$gene = $tmp[0];
	$net_feature{$gene} = join("\t",@tmp[2..10]);
}
close B;

my %het_trans;
my $het_header;
open(C,"$data_path/phase1_heterozygosity.txt")||die;
while(<C>){
	chomp $_;
	@tmp=split /\t/,$_;
	if (/^Transcript/){$het_header=join("\t",@tmp[1..$#tmp]);};
	$tr=$tmp[0];
	$het_trans{$tr}=join("\t",@tmp[1..$#tmp]);
}
close C;

my $exp_header;

my %exp_feature;
## read in expression data 
open(D,"$data_path/GTex_expression.txt")||die;
while(<D>){
	if (/^Adipose/){
		chomp $_;
		$exp_header = $_;
	}else{
		@tmp = split /\s+/,$_;
		$tr = $tmp[0];
		$exp_feature{$tr} = join("\t",@tmp[1..$#tmp]);
	}
}
close D;



print OUT join("\t","chr","pos","rsID","ref","alt","score","PASS","details","gene","gene_id","transcript_id","transcript_length","longest","partial","single_exon","NMD","canonical", "Gerp_Score", "in_Gerp_element","RS_score","percentage_gerp_elements_in_truncated_exons","truncated_exons/total_exons","segmental_duplications","PF","PFtruncated","SSF","SSFtruncated","SM","SMtruncated","Tmhmm","Tmhmmtruncated","Sigp","Sigptruncated","ACETYLATION","ACETYLATIONtruncated","DI-METHYLATION","DI-METHYLATIONtruncated","METHYLATION","METHYLATIONtruncated","MONO-METHYLATION","MONO-METHYLATIONtruncated","O-GlcNAc","O-GlcNActruncated","PHOSPHORYLATION","PHOSPHORYLATIONtruncated","SUMOYLATION","SUMOYLATIONtruncated","TRI-METHYLATION","TRI-METHYLATIONtruncated","UBIQUITINATION","UBIQUITINATIONtruncated","1000GPhase1","1000GPhase1_AF","1000GPhase1_ASN_AF","1000GPhase1_AFR_AF","1000GPhase1_EUR_AF","ESP6500","ESP6500_AAF","_pseudogenes_associated_to_transcript","_paralogs_associated_to_gene","dN/dS_(macaque)","dN/dS_(mouse)","shortest_path_to_recessive_gene","recessive_neighbors","shortest_path_to_dominant_gene","dominant_neighbors","shortest_path","recessive_dominant_neighbors","disorder_residue","disorder_residue_2","NSYn_density","SYn_density","average_Nsyn_Gerp_score","percent_of_Nsyn_in_Gerp_element","average_syn_Gerp_score","percent_of_syn_in_Gerp_element","miRNA_sites","Homozygous_Nonsynonymous","Homozygous_synonymous","Homozygous_prematureStop","Homozygous_spliceOverlap","Percentage_transcript_intersected_with_GERP_Elements"),"\t",$het_header,"\t",join("\t","PPI_DEGREE","METABOLIC_DEGREE","GENETIC_DEGREE","PHOS_DEGREE","ENCODE_NETS_DEGREE","SIGNALING_DEGREE","SUPERNET_DEGREE","NUM_NETWORKS","NUM_INTERFACES"),"\t",$exp_header,"\t","ExAC\tExAC_AF\n";
my $exac_af = &read_exac();
open(A,$input)||die;
while(<A>){
	if (/^chr\s+/){
		next;
	}
	
	chomp $_;
	s/N\/A/NA/g;
	s/1000GPhase1=//;
	s/1000GPhase1_AF=//;
	s/1000GPhase1_ASN_AF=//;
	s/1000GPhase1_AFR_AF=//;
	s/1000GPhase1_EUR_AF=//;
	s/ESP6500=//;
	s/ESP6500_AAF=//;
	
	@tmp=split /\t/,$_;
	#"chr","pos","gene","gene_id","transcript_id","transcript_length","longest","partial";
	print OUT join("\t",@tmp[0..7],$tmp[$#tmp-64],$tmp[$#tmp-63],$tmp[$#tmp-61],$tmp[$#tmp-60],$tmp[$#tmp-59],$tmp[$#tmp-62],$tmp[$#tmp-58], $tmp[$#tmp-55],$tmp[$#tmp-52],$tmp[$#tmp-48]),"\t";
	
	
	if ($tmp[$#tmp-47] eq "."){
		print OUT "0\tNA\t";
	}else{
		# in Gerp element; RS score
		print OUT "1\t";
		$tmp[$#tmp-47] =~ s/\)//g;
		$tmp[$#tmp-47] =~ s/\s+//g;
		print OUT (split /,/,$tmp[$#tmp-47])[2],"\t";
	}
	
	my ($tr_exon,$total_exon) = split /:/,$tmp[$#tmp-45];	
	$tr_exon =~ s/NA/0/;
	if ($tmp[$#tmp-46] eq "."){
		print OUT "NA\t",$tr_exon/$total_exon,"\t";
	}else{
		print OUT $tmp[$#tmp-46],"\t", $tr_exon/$total_exon,"\t";
	}
	
	if($tmp[$#tmp-44] eq "[]"){
		print OUT "0\t";
	}else{
		print OUT "1\t";
	}
	
	#protein domains & other
	
	print OUT join("\t",@tmp[$#tmp-42 ..$#tmp]),"\t";
	
	
	
	if ($tmp[$#tmp-3] ne "NA" && $tmp[$#tmp-1] ne "NA"){	
		if ($tmp[$#tmp-3] > $tmp[$#tmp-1]){
			print OUT $tmp[$#tmp-1],"\t";
		}else{
			print OUT $tmp[$#tmp-3],"\t";
		}
	
		print OUT $tmp[$#tmp-2]+$tmp[$#tmp],"\t";
	}else{
		print OUT "NA\tNA\t";
	}
	
	#disordered residues
	if ($tmp[$#tmp-43] eq "."){
		print OUT "0\t0\t";
	}else{
		print OUT join("\t", (split /\//,$tmp[$#tmp-43])[0,1]),"\t";
	}
	
	
	# transcript features
	if (defined $tr_feature{$tmp[$#tmp-61]}){
		print OUT $tr_feature{$tmp[$#tmp-61]},"\t";
	}else{
		print OUT "NA\t"x12; 
	}
	
	#heterozygosity
	if (defined $het_trans{$tmp[$#tmp-61]}){
		print OUT $het_trans{$tmp[$#tmp-61]},"\t";
	}else{
		print OUT "NA\t"x10;
	}

	# network degree
	if (defined $net_feature{$tmp[$#tmp-64]}){
		print OUT $net_feature{$tmp[$#tmp-64]},"\t";
	}else{
		print OUT "NA\t"x9;
	}

	# expression data
	my $tr = $tmp[$#tmp-61];

	if(defined $exp_feature{$tr}){
		print OUT $exp_feature{$tr},"\t";
	}else{
		print OUT "NA\t"x25,"NA\t";
	}
	$tmp[0]=~ s/chr//;
	
	my $chrPos = join('_',$tmp[0],$tmp[1],$tmp[3],$tmp[4]);
	if (defined $exac_af -> {$chrPos}){
		print OUT "1\t",$exac_af -> {$chrPos},"\n";
	}else{
		print OUT "0\t0\n";
	}
	
}	
close OUT;


sub read_exac{
        my $exac_af;
	open(A,"gunzip -c $data_path/ExAC.r0.3.sites.vcf.gz | ")||die;
	while(<A>){
                my @exac_tmp = split /\s+/,$_;
                $exac_af -> {join('_',@exac_tmp[0..3])} = $exac_tmp[4];

        }

        return($exac_af);
}

if (-s $tmp_out){
	`sed -i -e "s/\r//g" $tmp_out`;
	`Rscript $data_path/prediction.R $tmp_out $output $data_path`;
}
unlink ($tmp_out);
